@@ -3,7 +3,7 @@
  * The Horde_Auth_Base:: class provides a common abstracted interface to
  * creating various authentication backends.
  *
- * Copyright 1999-2011 Horde LLC (http://www.horde.org/)
+ * Copyright 1999-2013 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you did
  * not receive this file, http://www.horde.org/licenses/lgpl21
@@ -141,7 +141,13 @@ abstract class Horde_Auth_Base
             $this->_credentials['userId'] = $userId;
             if (($this->hasCapability('lock')) &&
                 $this->isLocked($userId)) {
-                throw new Horde_Auth_Exception('', Horde_Auth::REASON_LOCKED);
+                $details = $this->isLocked($userId, true);
+                if ($details['lock_timeout'] == Horde_Lock::PERMANENT) {
+                    $message = Horde_Auth_Translation::t("Your account has been permanently locked");
+                } else {
+                    $message = sprintf(Horde_Auth_Translation::t("Your account has been locked for %d minutes"), ceil(($details['lock_timeout'] - time()) / 60));
+                }
+                throw new Horde_Auth_Exception($message, Horde_Auth::REASON_LOCKED);
             }
             $this->_authenticate($userId, $credentials);
             $this->setCredential('userId', $this->_credentials['userId']);
@@ -157,7 +163,7 @@ abstract class Horde_Auth_Base
                     $this->hasCapability('badlogincount')) {
                     $this->_badLogin($userId);
                 }
-                $this->setError($code);
+                $this->setError($code, $e->getMessage());
             } else {
                 $this->setError(Horde_Auth::REASON_MESSAGE, $e->getMessage());
             }
@@ -226,8 +232,6 @@ abstract class Horde_Auth_Base
     /**
      * Locks a user indefinitely or for a specified time.
      *
-     * @since Horde_Auth 1.2.0
-     *
      * @param string $userId  The user to lock.
      * @param integer $time   The duration in minutes, 0 = permanent.
      *
@@ -240,8 +244,7 @@ abstract class Horde_Auth_Base
         }
 
         if ($time == 0) {
-            /* Roughly max timestamp32. */
-            $time = pow(2, 32) - time();
+            $time = Horde_Lock::PERMANENT;
         } else {
             $time *= 60;
         }
@@ -261,8 +264,6 @@ abstract class Horde_Auth_Base
     /**
      * Unlocks a user and optionally resets the bad login count.
      *
-     * @since Horde_Auth 1.2.0
-     *
      * @param string  $userId          The user to unlock.
      * @param boolean $resetBadLogins  Reset bad login counter?
      *
@@ -292,8 +293,6 @@ abstract class Horde_Auth_Base
     /**
      * Returns whether a user is currently locked.
      *
-     * @since Horde_Auth 1.2.0
-     *
      * @param string $userId         The user to check.
      * @param boolean $show_details  Return timeout too?
      *
@@ -328,8 +327,6 @@ abstract class Horde_Auth_Base
     /**
      * Handles a bad login.
      *
-     * @since Horde_Auth 1.2.0
-     *
      * @param string $userId  The user with a bad login.
      *
      * @throws Horde_Auth_Exception
@@ -347,7 +344,8 @@ abstract class Horde_Auth_Base
                 array('action' => 'login_failed', 'who' => $userId));
             $history_log = $this->_history_api->getHistory($history_identifier);
             if ($this->_params['login_block_count'] > 0 &&
-                $this->_params['login_block_count'] <= $history_log->count()) {
+                $this->_params['login_block_count'] <= $history_log->count() &&
+                $this->hasCapability('lock')) {
                 $this->lockUser($userId, $this->_params['login_block_time']);
             }
         } catch (Horde_History_Exception $e) {
@@ -358,8 +356,6 @@ abstract class Horde_Auth_Base
     /**
      * Resets the bad login counter.
      *
-     * @since Horde_Auth 1.2.0
-     *
      * @param string $userId  The user to reset.
      *
      * @throws Horde_Auth_Exception
